﻿//
// Copyright (c) 2009-2013, MindFusion LLC - Bulgaria.
//
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using MindFusion.Scheduling;
using System.Xml.Linq;

namespace MindFusion.Scheduling.Samples.CS.BusinessObjects
{
	/// <summary>
	/// Helper class that converts a CalendarEvent object to an Appointment instance and vice versa.
	/// </summary>
	public static class EventToAppointmentConverter
	{
		public static CalendarEvent CreateCalendarEvent(Appointment appointment)
		{
			if (appointment == null)
				throw new ArgumentNullException("appointment", "appointment cannot be null");
			return new CalendarEvent
				(
					new Guid(appointment.Tag.ToString()),
					appointment.HeaderText,
					appointment.Details,
					appointment.StartTime,
					appointment.EndTime - appointment.StartTime
				);
		}

		public static Appointment CreateAppointment(CalendarEvent calendarEvent)
		{
			if (calendarEvent == null)
				throw new ArgumentNullException("calendarEvent", "calendarEvent cannot be null");
			return new Appointment() {
				HeaderText = calendarEvent.Name,
				Details = calendarEvent.Details,
				StartTime = calendarEvent.StartTime,
				EndTime = calendarEvent.StartTime + calendarEvent.Duration,
				Tag = calendarEvent.Id
			};
		}

		public static XElement ConvertToXElement(CalendarEvent calendarEvent)
		{
			var xml =
				new XElement("CalendarEvent",
					new XElement("Id", calendarEvent.Id),
					new XElement("Name", calendarEvent.Name),
					new XElement("Details", calendarEvent.Details),
					new XElement("StartTime", calendarEvent.StartTime),
					new XElement("Duration", calendarEvent.Duration.ToString())
					);
			return xml;
		}

		public static CalendarEvent ConvertFromXElement(XElement element)
		{
			Guid id = new Guid(element.Element("Id").Value);
			string name = element.Element("Name").Value;
			string details = element.Element("Details").Value;
			DateTime startTime = DateTime.Parse(element.Element("StartTime").Value);
			TimeSpan duration = TimeSpan.Parse(element.Element("Duration").Value);
			return new CalendarEvent(id, name, details, startTime, duration);
		}
	}
}
