﻿//
// Copyright (c) 2009-2013, MindFusion LLC - Bulgaria.
//
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using MindFusion.Scheduling;
using MindFusion.Scheduling.WebForms;
using System.Xml.Linq;

namespace MindFusion.Scheduling.Samples.CS.BusinessObjects
{
	public partial class _Default : System.Web.UI.Page
	{
		private DateTime start = new DateTime(DateTime.Today.Year, DateTime.Today.Month, 1);
		protected void Page_Load(object sender, EventArgs e)
		{
			if (!IsPostBack)
			{
				Calendar1.Date = start;
				Calendar1.EndDate = Calendar1.Date.AddMonths(1);
				// Populate the Schedule
				InitializeAppointments();
			}
		}

		/// <summary>
		/// Handles the Calendar.ItemCreated event.
		/// </summary>
		protected void Calendar1_ItemCreated(object sender, ItemEventArgs e)
		{
			CreateEvent(e.Item as Appointment);
			InitializeAppointments();
		}

		/// <summary>
		/// Handles the Calendar.ItemDeleted event.
		/// </summary>
		protected void Calendar1_ItemDeleted(object sender, ItemEventArgs e)
		{
			DeleteEvent(e.Item as Appointment);
			InitializeAppointments();
		}

		/// <summary>
		/// Handles the Calendar.ItemModified event.
		/// </summary>
		protected void Calendar1_ItemModified(object sender, ItemModifiedEventArgs e)
		{
			EditEvent(e.Item as Appointment);
			InitializeAppointments();
		}

		/// <summary>
		/// Handles the Calendar.ItemInplaceEdited event.
		/// </summary>
		protected void Calendar1_ItemInplaceEdited(object sender, ItemEventArgs e)
		{
			EditEvent(e.Item as Appointment);
			InitializeAppointments();
		}

		/// <summary>
		/// Create a new CalendarEvent object from an Appointment object and update the undelrying XML data store.
		/// </summary>
		/// <param name="appointment">
		///	the Appoinement object to create the CalendarEvent from.
		/// </param>
		private void CreateEvent(Appointment appointment)
		{
			appointment.Tag = new Guid(appointment.Id.Substring(3));
			CalendarEvent newEvent = EventToAppointmentConverter.CreateCalendarEvent(appointment);

			XElement events = XElement.Load(Server.MapPath(".") + "\\events.xml");
			events.Add(EventToAppointmentConverter.ConvertToXElement(newEvent));
			events.Save(Server.MapPath(".") + "\\events.xml");
		}

		/// <summary>
		/// Delete a CalendarEvent object from the undelrying XML data store.
		/// The CalendarEvent to delete is identified by the Appointment object.
		/// </summary>
		/// <param name="appointment">
		///	the Appoinement object to identify the CalendarEvent with.
		/// </param>
		private void DeleteEvent(Appointment appointment)
		{
			Guid id = new Guid(appointment.Tag.ToString());

			XElement events = XElement.Load(Server.MapPath(".") + "\\events.xml");
			XElement deleted = events.Descendants("CalendarEvent")
				.Where(x => x.Descendants("Id").SingleOrDefault().Value == id.ToString()).SingleOrDefault();
			if (deleted != null)
				deleted.Remove();

			events.Save(Server.MapPath(".") + "\\events.xml");
		}

		/// <summary>
		/// Delete a CalendarEvent object in the undelrying XML data store.
		/// The CalendarEvent to modify is identified by the Appointment object.
		/// </summary>
		/// <param name="appointment">
		///	the Appoinement object to identify the CalendarEvent with.
		/// </param>
		private void EditEvent(Appointment appointment)
		{
			string id = appointment.Tag.ToString();
			CalendarEvent editedEvent = new CalendarEvent();
			editedEvent.CopyFrom(EventToAppointmentConverter.CreateCalendarEvent(appointment));
			XElement events = XElement.Load(Server.MapPath(".") + "\\events.xml");
			XElement edited = events.Descendants("CalendarEvent")
				.Where(x => x.Descendants("Id").SingleOrDefault().Value == id).SingleOrDefault();
			if (edited != null)
				edited.ReplaceWith(EventToAppointmentConverter.ConvertToXElement(editedEvent));

			events.Save(Server.MapPath(".") + "\\events.xml");
		}

		/// <summary>
		/// Synchronize the Schedule with the underlying XML data store.
		/// </summary>
		private void InitializeAppointments()
		{
			Calendar1.Schedule.Clear();
			if (!System.IO.File.Exists(Server.MapPath(".") + "\\events.xml"))
				RecreateFile();
			var query = from c in XElement.Load(Server.MapPath(".") + "\\events.xml").Elements()
						select c;
			foreach (var evt in query)
			{
				CalendarEvent calEvt = EventToAppointmentConverter.ConvertFromXElement(evt);
				Appointment app = EventToAppointmentConverter.CreateAppointment(calEvt);
				Calendar1.Schedule.Items.Add(app);
			}
		}

		/// <summary>
		/// Recreate the XML data store.
		/// </summary>
		private void RecreateFile()
		{
			List<CalendarEvent> tempStore = new List<CalendarEvent>();

			CalendarEvent evt = new CalendarEvent()
			{
				Id = Guid.NewGuid(),
				Name = "Have Fun!",
				Details = "Work hard, but sometimes we need to rest.",
				StartTime = start + TimeSpan.FromDays(8) + TimeSpan.FromHours(10),
				Duration = TimeSpan.FromHours(6)
			};
			tempStore.Add(evt);

			evt = new CalendarEvent()
			{
				Id = Guid.NewGuid(),
				Name = "Lunch with Laura",
				Details = "Never to forget the appointment with the boss.",
				StartTime = start + TimeSpan.FromHours(12),
				Duration = TimeSpan.FromHours(3.5)
			};
			tempStore.Add(evt);

			evt = new CalendarEvent()
			{
				Id = Guid.NewGuid(),
				Name = "Get the children from the kindergarten!",
				Details = "Yea!",
				StartTime = start + TimeSpan.FromDays(7) + TimeSpan.FromHours(17),
				Duration = TimeSpan.FromHours(0.45)
			};
			tempStore.Add(evt);

			evt = new CalendarEvent()
			{
				Id = Guid.NewGuid(),
				Name = "William's birthday",
				Details = "Party!",
				StartTime = start + TimeSpan.FromDays(1),
				Duration = TimeSpan.FromDays(1)
			};
			tempStore.Add(evt);

			evt = new CalendarEvent()
			{
				Id = Guid.NewGuid(),
				Name = "Visit the dentist",
				Details = "This reminder is rather pointless, since my tooth is a reminder on its own",
				StartTime = start + TimeSpan.FromDays(9) + TimeSpan.FromDays(2),
				Duration = TimeSpan.FromDays(1)
			};
			tempStore.Add(evt);

			evt = new CalendarEvent()
			{
				Id = Guid.NewGuid(),
				Name = "Arrange the house payments!",
				Details = "Yea!",
				StartTime = start + TimeSpan.FromDays(2) + TimeSpan.FromHours(17.5),
				Duration = TimeSpan.FromHours(3.5)
			};
			tempStore.Add(evt);

			evt = new CalendarEvent()
			{
				Id = Guid.NewGuid(),
				Name = "Meet George",
				Details = "This is the seventh time I promise George. This time I should go.",
				StartTime = start - TimeSpan.FromDays(-10) + TimeSpan.FromHours(8.5),
				Duration = TimeSpan.FromHours(1)
			};
			tempStore.Add(evt);

			evt = new CalendarEvent()
			{
				Id = Guid.NewGuid(),
				Name = "Prepare the dinner!",
				Details = "Yea!",
				StartTime = start + TimeSpan.FromDays(12) + TimeSpan.FromHours(18),
				Duration = TimeSpan.FromHours(1)
			};
			tempStore.Add(evt);

			XElement list =
				new XElement("CalendarEvents",
					from item in tempStore
					select EventToAppointmentConverter.ConvertToXElement(item));
			list.Save(Server.MapPath(".") + "\\events.xml");
		}
	}
}
